<?php

/**
 * @file
 * Contains all picture management functions for Gallerix, and all settings pages.
 */

/**
 * Initialize and return the gallerix menu items.
 *
 * @return An array of menu items.
 */
function _gallerix_menu_items() {
  $items = array();

  $items['admin/settings/gallerix'] = array(
    'title' => 'Gallerix',
    'type' => MENU_LOCAL_TASK,
    'description' => 'Administer Gallerix Settings.',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('gallerix_admin_settings'),
    'access arguments' => array('configure gallerix'),
    'file' => 'gallerix.admin.inc',
    'weight' => -1,
  );
  $items['admin/settings/gallerix/general'] = array(
    'title' => 'General Settings',
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'description' => 'Configure your galleries.',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('gallerix_admin_settings'),
    'access arguments' => array('configure gallerix'),
    'file' => 'gallerix.admin.inc',
    'weight' => -1,
  );
  $items['admin/settings/gallerix/display'] = array(
    'title' => 'Display Settings',
    'type' => MENU_LOCAL_TASK,
    'description' => 'Change display options.',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('gallerix_display_settings'),
    'access arguments' => array('configure gallerix'),
    'file' => 'gallerix.admin.inc',
  );
  $items['admin/settings/gallerix/rebuild'] = array(
    'title' => 'Rebuild Pictures',
    'type' => MENU_LOCAL_TASK,
    'description' => "Rebuild the entire picture collection on this site.",
    'page callback' => 'gallerix_rebuild',
    'access arguments' => array('configure gallerix'),
    'file' => 'gallerix.build.inc',
  );

  // AJAX Rebuild Functions
  $items['admin/settings/gallerix/rebuild/build'] = array(
    'page callback' => 'gallerix_rebuild_build',
    'type' => MENU_CALLBACK,
    'access arguments' => array('configure gallerix'),
    'file' => 'gallerix.build.inc',
  );
  $items['admin/settings/gallerix/rebuild/finalize'] = array(
    'page callback' => 'gallerix_rebuild_finalize',
    'type' => MENU_CALLBACK,
    'access arguments' => array('configure gallerix'),
    'file' => 'gallerix.build.inc',
  );

  $items['node/%node/pictures'] = array(
    'title' => 'Pictures',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('gallerix_add_pictures_form', 1),
    'type' => MENU_LOCAL_TASK,
    'access callback' => '_gallerix_menu_access',
    'access arguments' => array(1),
    'file' => 'gallerix.admin.inc',
  );
  $items['node/%node/pictures/add'] = array(
    'title' => 'Add',
    'type' => MENU_DEFAULT_LOCAL_TASK,
    'access callback' => '_gallerix_menu_access',
    'access arguments' => array(1),
  );

  $items['node/%node/pictures/manage'] = array(
    'title' => 'Manage',
    'page callback' => 'drupal_get_form',
    'page arguments' => array('gallerix_manage_pictures_form', 1),
    'type' => MENU_LOCAL_TASK,
    'access callback' => '_gallerix_menu_access',
    'access arguments' => array(1, 'manage'),
    'file' => 'gallerix.admin.inc',
  );
  $items['node/%node/pictures/manage/submit'] = array(
    'page callback' => 'gallerix_manage_form_submit',
    'type' => MENU_CALLBACK,
    'access callback' => '_gallerix_menu_access',
    'access arguments' => array(1, 'manage'),
    'file' => 'gallerix.admin.inc',
  );
  $items['node/%node/pictures/manage/delete'] = array(
    'page callback' => 'gallerix_manage_form_delete',
    'type' => MENU_CALLBACK,
    'access callback' => '_gallerix_menu_access',
    'access arguments' => array(1, 'manage'),
    'file' => 'gallerix.admin.inc',
  );

  // Author Album Building.
  $items['node/%node/pictures/build'] = array(
    'page callback' => 'gallerix_build_album',
    'page arguments' => array(1),
    'title' => t('Build'),
    'type' => MENU_LOCAL_TASK,
    'access callback' => '_gallerix_menu_access',
    'access arguments' => array(1, 'build'),
    'file' => 'gallerix.build.inc',
  );
  // Author AJAX callback
  $items['node/%node/pictures/build/picture'] = array(
    'page callback' => 'gallerix_build_album_picture',
    'type' => MENU_CALLBACK,
    'access callback' => '_gallerix_menu_access',
    'access arguments' => array(1),
    'file' => 'gallerix.build.inc',
  );
  // Author AJAX callback
  // Used by the finalize form.
  $items['node/%node/pictures/build/finalize'] = array(
    'type' => MENU_CALLBACK,
    'access callback' => '_gallerix_menu_access',
    'access arguments' => array(1),
    'file' => 'gallerix.build.inc',
  );


  // Users AJAX Functions
  $items['gallerix/grid'] = array(
    'page callback' => 'gallerix_fetch_grid',
    'type' => MENU_CALLBACK,
    'access arguments' => array('access content'),
  );
  $items['gallerix/picture'] = array(
    'page callback' => 'gallerix_fetch_picture',
    'type' => MENU_CALLBACK,
    'access arguments' => array('access content'),
  );

  return $items;
}

/**
 * Provides a display settings page.
 *
 * @return The gallerix settings form in an array.
 */
function gallerix_display_settings() {
  // Display Options
  $form['gallerix_display'] = array(
    '#type' => 'fieldset',
    '#title' => t('Display Options'),
  );
  $form['gallerix_display']['gallerix_default_view'] = array(
    '#type' => 'select',
    '#title' => t('Default View'),
    '#description' => t("Select which view to default to. Grid view will display all the images in an album. Single View will display the first image of the album, along with thumbnails if enabled below. A link is provided to switch between the two."),
    '#options' => array('grid' => 'Grid', 'viewer' => 'Single'),
    '#default_value' => variable_get('gallerix_default_view', 'grid'),
  );
  $form['gallerix_display']['gallerix_default_sort'] = array(
    '#type' => 'select',
    '#title' => t('Default Sort Order'),
    '#description' => t("Your album pictures will be displayed in the selected order. You can override this setting individually by selecting a different order in the album creation page."),
    '#options' => array('ascending' => t('Date Taken Ascending'), 'descending' => t('Date Taken Descending'), 'random' => t('Random'), 'filename' => t('Filename'), 'custom' => t('Custom')),
    '#default_value' => variable_get('gallerix_default_sort', 'ascending'),
  );
  $form['gallerix_display']['gallerix_thumbnail_number'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of Thumbnails'),
    '#description' => t('This number controls the amount of thumbnails displayed in each album for browsing. Raising this might decrease performance since there will be more database queries.'),
    '#size' => 2,
    '#default_value' => variable_get('gallerix_thumbnail_number', 5),
  );
  $form['gallerix_display']['gallerix_translucence'] = array(
    '#type' => 'checkbox',
    '#title' => t("Thumbnail Translucence"),
    '#description' => t('When enabled, thumbnails will be made translucent until the mouse hovers over them.'),
    '#default_value' => variable_get('gallerix_translucence', 1),
  );

  // Layout options
  $form['gallerix_layout'] = array(
    '#type' => 'fieldset',
    '#tree' => TRUE,
    '#title' => t('Layout Options'),
    '#theme' => 'gallerix_component_settings'
  );

  $components = _gallerix_load_components('configure');

  foreach ($components as $component) {
    $form['gallerix_layout'][$component['#id']]['title'] = array(
      '#type' => 'value',
      '#value' => $component['#title'],
    );
    $form['gallerix_layout'][$component['#id']]['weight'] = array(
      '#type' => 'weight',
      '#delta' => 5,
      '#title' => $component['#title'],
      '#default_value' => $component['#weight'],
    );
    $form['gallerix_layout'][$component['#id']]['display'] = array(
      '#type' => 'checkbox',
      '#title' => t("Enabled"),
      '#default_value' =>  $component['#display'],
    );
  }

  $form['gallerix_style'] = array(
    '#type' => 'fieldset',
    '#title' => t('Style Options'),
  );
  $form['gallerix_style']['gallerix_style'] = array(
    '#type' => 'select',
    '#title' => t('CSS Stylesheet'),
    '#description' => t('Select a style to use for Gallerix. Two styles are included by default, but select custom if you wish to use your own stylesheet'),
    '#options' => array('standard.css' => t('Standard'), 'edgeless.css' => t('Edgeless'), 'custom' => t('Custom')),
    '#default_value' => variable_get('gallerix_style', 'standard.css'),
  );
  $form['gallerix_style']['gallerix_style_path'] = array(
    '#type' => 'textfield',
    '#title' => t('Custom Stylesheet Path'),
    '#description' => t("If you choose to use your own stylesheet, enter the path where the stylesheet can be found. Don't forget that icons need to be relative to the stylesheet,
                         so copy those over if necesssary, unless you are providing your own."),
    '#disabled' => variable_get('gallerix_style', 'standard.css') != 'custom',
    '#default_value' => variable_get('gallerix_style_path', ''),
  );


  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit')
  );

  return $form;
}

function theme_gallerix_component_settings($form) {

  $header = array(t('Component'), t('Weight'), t('Display'));
  $rows = array();

  foreach (element_children($form) as $key) {
    //Remove titles from the select boxes.
    unset($form[$key]['weight']['#title']);
    unset($form[$key]['display']['#title']);

    //Extrac the title for each component, then unset it.
    $title = $form[$key]['title']['#value'];
    unset($form[$key]['title']);


    $rows[] = array(
      $title,
      drupal_render($form[$key]['weight']),
      drupal_render($form[$key]['display']),
    );
  }

  return theme('table', $header, $rows);
}


function gallerix_display_settings_validate($form, &$form_state) {

  if ($form_state['values']['gallerix_thumbnail_number'] != strval(intval($form_state['values']['gallerix_thumbnail_number']))) {
    form_set_error('gallerix_thumbnail_number', t('You must enter an number.'));
  }
  elseif (intval($form_state['values']['gallerix_thumbnail_number']) == 1 || intval($form_state['values']['gallerix_thumbnail_number']) == 2 || intval($form_state['values']['gallerix_thumbnail_number']) < 0) {
    form_set_error('gallerix_thumbnail_number', t('You must use at least 3 thumbnails. You may set to 0 to disable thumbnails.'));
  }

}

function gallerix_display_settings_submit($form, &$form_state) {

  $weights = array();
  $display = array();

  foreach ($form_state['values']['gallerix_layout'] as $key => $component) {
    $weights[$key] = $component['weight'];
    $display[$key] = $component['display'];
  }


  unset($form_state['values']['gallerix_layout']);

  variable_set('gallerix_layout_weights', $weights);
  variable_set('gallerix_layout_display', $display);

  cache_clear_all(NULL, 'cache_gallerix');

  system_settings_form_submit($form, $form_state);
}



function gallerix_admin_settings() {
  $sizes = gallerix_sizes();

  // Dummy to add a blank at the bottom of the table.
  $sizes[''] = array(
    'label' => '',
    'name' => '',
    'width' => '',
    'height' => '',
    'ratio' => 0,
  );

  // Resolution Options
  $form['gallerix_sizes'] = array(
    '#type' => 'fieldset',
    '#title' => t('Resolutions'),
    '#theme' => 'gallerix_sizes_table',
    '#tree' => TRUE,
    '#description' => t("You may add more sizes to build for each picture submitted. If you want the picture to be resized exactly to the proportions,
                         check the Maintain Ratio box. If you don't check the box, pictures will only be scaled to fit given proportions."),
  );

  foreach ($sizes as $name => $size) {
    $entry = array();
    $entry['label'] = array(
      '#type' => 'textfield',
      '#default_value' => $size['label'],
      '#size' => 20,
    );
    $disabled = $name == 'thumbnail' || $name == 'frame';
    $entry['name'] = array(
      '#type' => 'textfield',
      '#size' => 20,
      ($disabled ? '#value' : '#default_value') => $name,
      '#disabled' => $disabled,
    );
    $entry['width'] = array(
      '#type' => 'textfield',
      '#size' => 5,
      '#default_value' => $size['width'],
    );
    $entry['height'] = array(
      '#type' => 'textfield',
      '#size' => 5,
      '#default_value' => $size['height'],
    );
    $entry['ratio'] = array(
      '#type' => 'checkbox',
      '#default_value' => $size['ratio'],
    );

    $form['gallerix_sizes'][] = $entry;

  }

  // Source Options
  $form['gallerix_sources'] = array(
    '#type' => 'fieldset',
    '#title' => t('Picture Sources'),
  );
  $form['gallerix_sources']['gallerix_upload_number'] = array(
    '#type' => 'textfield',
    '#title' => t('Number of Upload Fields'),
    '#description' => t('Controls the number of file upload fields available to users at the same time. Set to 0 to disable file upload.'),
    '#size' => 2,
    '#default_value' => variable_get('gallerix_upload_number', 1),
  );
  $form['gallerix_sources']['gallerix_allow_archives'] = array(
    '#type' => 'checkbox',
    '#title' => t('Allow ZIP'),
    '#disabled' => !extension_loaded ('zip'),
    '#description' => t('When enabled, users will be able to upload ZIP archive files as well as images. This option is disabled if your PHP installation does not have the ZIP library.'),
    '#size' => 2,
    '#default_value' => variable_get('gallerix_allow_archives', FALSE),
  );
  $form['gallerix_sources']['gallerix_repository'] = array(
    '#type' => 'checkbox',
    '#title' => t('Enable Repository.'),
    '#description' => t('Enabling the repository will provide a common place to upload your pictures using FTP.
                         This is very useful if you are not using per-user albums, or you have a site with a controlled set of a few trusted users.
                         When enabled, simply place picture files inside files/gallerix/repository, and they will show up whenever you are adding pictures to your albums. '),
    '#default_value' => variable_get('gallerix_repository', 0),
  );
  $form['gallerix_sources']['gallerix_delete_repository'] = array(
    '#type' => 'checkbox',
    '#title' => t('Delete From Repository.'),
    '#description' => t('When enabled, files are moved from the repository to the album folder. If disabled, a copy is made instead and the original files are left untouched in the repository folder.'),
    '#default_value' => variable_get('gallerix_delete_repository', FALSE)
  );


  // Insertion Options
  $form['gallerix_metadata'] = array(
    '#type' => 'fieldset',
    '#title' => t('Metadata Options'),
  );
  $form['gallerix_metadata']['gallerix_extract_exif'] = array(
    '#type' => 'checkbox',
    '#title' => t('Extract EXIF Data'),
    '#disabled' => !function_exists('exif_read_data'),
    '#description' => t('Information such as Date Taken and Image Description are stored inside each file. If you enable this option, Gallerix will try to extract EXIF data from pictures.
                         Sorting by dates only works if this feature is enabled. Note that this feature is automatically disabled if your PHP installation does not support EXIF extraction.'),
    '#default_value' => variable_get('gallerix_extract_exif', 0),
  );
  $form['gallerix_metadata']['gallerix_extract_iptc'] = array(
    '#type' => 'checkbox',
    '#title' => t('Extract IPTC Tags'),
    '#description' => t("Use this option to have Gallerix attempt to extract IPTC keywords from the files. This is very useful if you use Photoshop Elements or Photoshop to tag your files.
                         Captions for pictures will be extracted from IPTC data and linked to pictures. Users can then override this caption if they wish to do so."),
    '#default_value' => variable_get('gallerix_extract_iptc', 0),
  );

  // Security Options
  $form['gallerix_security'] = array(
    '#type' => 'fieldset',
    '#title' => t('Gallerix Security'),
  );
  $form['gallerix_security']['gallerix_secure_mode'] = array(
    '#type' => 'checkbox',
    '#title' => t('Secure Mode'),
    '#description' => t('Overall, secure mode tries to ensure that users only view pictures that belong to albums they are allowed to see. Remember, actual album access is not controlled. Use a Node Access module for this. Also, if you are using the Public Upload folder, such a security is really not secure.')
      . (variable_get('file_downloads', FILE_DOWNLOADS_PUBLIC) == FILE_DOWNLOADS_PUBLIC ?
          '<br /><span style="color: red;">' . t('WARNING:') . '</span> '
          . t('the !filesystem is currently PUBLIC; therefore this protection will not help very much in really protecting your images.',
              array('!filesystem' => l('file system', 'admin/settings/file-system'))) : ''),
    '#default_value' => variable_get('gallerix_secure_mode', 0),
  );

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  return $form;
}


function theme_gallerix_sizes_table($form) {

  $header = array(t('Label'), t('Name'), t('Width'), t('Height'), t('Maintain Ratio'));
  $rows = array();

  foreach (element_children($form) as $key) {
    $rows[] = array(
      drupal_render($form[$key]['label']),
      drupal_render($form[$key]['name']),
      drupal_render($form[$key]['width']),
      drupal_render($form[$key]['height']),
      drupal_render($form[$key]['ratio']),
    );
  }

  return theme('table',  $header, $rows);
}





function gallerix_admin_settings_validate($form_id, &$form_state) {
  // $names is used to verify that each name is used only once
  $names = array();
  foreach ($form_state['values']['gallerix_sizes'] as $key => $size) {
    if ($size['name'] && $size['label']) {
      if (in_array($size['name'], $names)) {
        form_set_error('gallerix_sizes][' . $key . '][name', t('The name is already taken.'));
      }
      elseif (!preg_match("|^[a-z]+$|", $size['name'])) {
        form_set_error('gallerix_sizes][' . $key . '][name', t('Names can only contain lowercase letters (a-z). No spaces, underscores, or dashes.'));
      }
      else {
        $names[] = $size['name'];
      }
      if (!preg_match("|^[0-9]+$|", $size['width']) || $size['width'] <= 0) {
        form_set_error('gallerix_sizes][' . $key . '][width', t('Width must be a positive number.'));
      }
      if (!preg_match("|^[0-9]+$|", $size['height']) || $size['height'] <= 0) {
        form_set_error('gallerix_sizes][' . $key . '][height', t('Height must be a positive number.'));
      }
    }
    elseif ($size['name']) {
      form_set_error('gallerix_sizes][' . $key . '][label', t('A label is required.'));
    }
    elseif ($size['label']) {
      form_set_error('gallerix_sizes][' . $key . '][name', t('An internal name is required.'));
    }
  }
}



function gallerix_admin_settings_submit($form_id, &$form_state) {
  foreach ($form_state['values']['gallerix_sizes'] as $key => $size) {
    if ($size['name']) {
      $name = $size['name'];
      unset($size['name']);
      $sizes[$name] = $size;
    }
  }
  variable_set('gallerix_sizes', $sizes);

  // some cleanup to make sure the users see the changes immediately
  unset($form_state['values']['gallerix_sizes']);
  cache_clear_all(NULL, 'cache_gallerix');
  system_settings_form_submit($form_id, $form_state);
}







/**
 * Provides means to upload pictures into an album.
 *
 */
function gallerix_add_pictures_form(&$form_state, $node) {

  drupal_set_title(t('Add Pictures'));

  // Repositories.
  $common_repository = file_directory_path() . '/gallerix/repository' ;

  $upload_files = variable_get('gallerix_upload_number', 1);
  $archive_allowed = variable_get('gallerix_allow_archives', FALSE);

  $extensions = $archive_allowed ? '|ZIP|zip' : '';
  $allowed_files = '.+\.(jpe?g|png|gif|JPE?G|PNG|GIF' . $extensions . ')';

  $common_files = variable_get('gallerix_repository', 0) ? file_scan_directory($common_repository, $allowed_files) : 0;


  $album_pictures = db_query('SELECT pid, path, name, caption, original_name FROM {gallerix_pictures} WHERE nid = %d ORDER BY sort', $node->nid);

  $form = array();
  $form['#tree'] = TRUE;

  // We use this as a placeholder to tell the submit function what to do.
  $actions = array();

  // We want to pass the node ID.
  $form['node_id'] = array(
    '#type' => 'value',
    '#value' => $node->nid,
  );
  $form['#attributes'] = array("enctype" => "multipart/form-data");

  // Manual Upload
  if ($upload_files) {
    $form['gallerix_upload'] = array(
      '#type' => 'fieldset',
      '#title' => t('From Upload'),
      '#tree' => FALSE,
    );

    for ($x = 1; $x <= $upload_files; $x++) {
      if ($upload_files == 1) {
        $upload_text = t('Upload File');
      }
      else {
        $upload_text = t('Upload File !field_number', array('!field_number' => $x));
      }

      $form['gallerix_upload']['upload_' . $x] = array(
        '#type' => 'file',
        '#title' => $upload_text,
        '#size' => 40,
        '#description' => t('Select an image to upload.') . ($archive_allowed ? t(' You may also upload a ZIP file.') : ''),
        '#default_value' => '',
      );
    }
  }
  // Only display the "Add From Common Repository" form if there are any files to add.
  if ($common_files) {
    $actions['repository'] = TRUE; //Process repository.
    foreach ($common_files as $file) {
      $form['gallerix_repository']['checkboxes'][] = array(
        '#type' => 'checkbox',
        '#title' => check_plain($file->basename),
        '#collapsible' => TRUE,
      );
      $form['gallerix_repository']['files'][] = array(
        '#type' => 'value',
        '#value' => array('filename' => $file->filename, 'basename' => $file->basename, 'filesize' => filesize($file->filename)),
      );
    }
  }


  // Let the submit function know what to process.
  $form['actions'] = array(
    '#type' => 'value',
    '#value' => $actions,
  );


  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );


  return $form;
}



/**
 * Themes the add form into a table.
 *
 */
function theme_gallerix_add_pictures_form($form) {

  $node = node_load($form['node_id']['#value']);
  $upload_files = variable_get('gallerix_upload_number', 1);
  $paths = _gallerix_album_paths($node, TRUE);

  $header = array(theme('table_select_header_cell'), t('Picture'), t('Caption'));
  $rows = array();

  $header = array(theme('table_select_header_cell'), t('File Name'), t('File Size'));

  if (!empty($form['gallerix_repository']['files'])) {
    foreach (element_children($form['gallerix_repository']['files']) as $key) {
      $basename = $form['gallerix_repository']['files'][$key]['#value']['basename'];
      $filesize = $form['gallerix_repository']['files'][$key]['#value']['filesize'];
      $filesize = format_size($filesize);

      unset($form['gallerix_repository']['checkboxes'][$key]['#title']);

      $rows[] = array(
        drupal_render($form['gallerix_repository']['checkboxes'][$key]),
        $basename,
        $filesize
      );
    }
  }

  $output = '';

  if (!empty($form['actions']['#value']['repository'])) {
    $output .= '<h3>Repository Files</h3>';
    $output .= theme('table', $header, $rows);
  }

  $output .= drupal_render($form['submit']);

  //Put the upload field on top.
  return drupal_render($form) . $output;
}



/**
 * Figure out what's been uploaded and update the album with the new pictures.
 *
 */
function gallerix_add_pictures_form_submit($form_id, &$form_state) {

  $node = node_load($form_state['values']['node_id']);
  $paths = _gallerix_album_paths($node, TRUE);

  $album_directory = $paths['album'];

  $rebuild = FALSE;
  $rebuild = FALSE;

  // Load in the settings.
  $extract_exif = variable_get('gallerix_extract_exif', 0);
  $fields_to_extract = $extract_exif ? variable_get('gallerix_fields_to_extract', 0) : array();
  $delete_repository_files = variable_get('gallerix_delete_repository', FALSE);
  $upload_files = variable_get('gallerix_upload_number', 1);
  $archive_allowed = variable_get('gallerix_allow_archives', FALSE);

  $files = array();

  if ($upload_files) {
    for ($x = 1; $x <= $upload_files; $x++) {


      if ($file = file_save_upload('upload_' . $x, array(), file_directory_path(), FILE_EXISTS_RENAME)) {

        // We don't want to use Drupal's file system.
        db_query('DELETE FROM {files} WHERE fid = %d', $file->fid);

        $temp_files = gallerix_process_file($file->filepath, $paths['original'], TRUE);
        if ($temp_files) {
          $files = array_merge($temp_files, $files);
        }
        else {
          file_delete($file->filepath);
        }
      }
    }
  }

  // Move files from repository to album.
  if (!empty($form_state['values']['actions']['repository'])) {
    foreach ($form_state['values']['gallerix_repository']['checkboxes'] as $key => $checked) {
      if ($checked) {
        $source = $form_state['values']['gallerix_repository']['files'][$key]['filename'];
        $destination = $paths['original'] . '/' . $form_state['values']['gallerix_repository']['files'][$key]['basename'];

        $file = new stdClass();
        $file->filename = $form_state['values']['gallerix_repository']['files'][$key]['basename'];
        $file->filepath = $source;

        $temp_files = gallerix_process_file($source, $paths['original'], $delete_repository_files);
        if ($temp_files) {
          $files = array_merge($temp_files, $files);
        }
      }
    }
  }

  // For each added file, make a new entry in the database.
  $sort = 1;
  foreach ($files as $file) {
    $original_name = basename($file);

    $secure_path = _gallerix_secure_file($file);
    $secure_name = basename($secure_path);

    db_query("INSERT INTO {gallerix_pictures}"
        . " (nid, uid, path, name, original_name, added, created, caption, sort)"
        . " VALUES (%d, %d, '%s', '%s', '%s', %d, %d, '%s', %d)",
      $node->nid, $node->uid, $secure_path, $secure_name, $original_name, time(), time(), '', $sort);
    ++$sort;
  }

  cache_clear_all(NULL, 'cache_gallerix');

  if (count($files)) {
    $form_state['redirect'] = 'node/' . $node->nid . '/pictures/build';
  }
}




/**
 * Provides an AJAX based management form for pictures in an album.
 *
 * @param[in,out] $form_state The current state of the form.
 * @param[in] $node The album node being managed.
 *
 * @return The picture management form as an array.
 */
function gallerix_manage_pictures_form(&$form_state, $node) {
  drupal_set_title(t('Manage Pictures'));

  //Load common.js.
  gallerix_load_js();

  //Include management JavaScript and CSS.
  $path = drupal_get_path('module', 'gallerix');
  drupal_add_js($path . '/js/admin.js');
  drupal_add_css($path . '/css/admin.css');

  $album_pictures = db_query('SELECT pid, path, name, caption, original_name FROM {gallerix_pictures} WHERE nid = %d ORDER BY sort', $node->nid);

  $form = array();
  $form['#tree'] = TRUE;

  // We use this as a placeholder to tell the submit function what to do.
  $actions = array();

  // We want to pass the node ID.
  $form['node_id'] = array(
    '#type' => 'value',
    '#value' => $node->nid,
  );

  $count = 0;

  $index = array();
  $pictures = array();
  $captions = array();

  $sort = $node->gallerix['sort'];
  $sort = ($sort == 'default') ? variable_get('gallerix_default_sort', 'ascending') : $sort;


  while ($picture = db_fetch_object($album_pictures)) {
    $actions['album'] = TRUE; // Process album.

    $form['gallerix_album']['files'][] = array(
      '#type' => 'value',
      '#value' => array(
        'filename' => $picture->path,
        'basename' => $picture->name,
        'pid' => $picture->pid,
        'caption' => $picture->caption,
        'original_name' => $picture->original_name,
      ),
    );
    $form['gallerix_album']['captions'][] = array(
      '#type' => 'textfield',
      '#title' => 'Caption',
      '#id' => 'gallerix-caption-field-' . $picture->pid,
      '#attributes' => array('class' => 'gallerix-caption-field'),
    );

    $index[] = $picture->pid;
    $pictures[$picture->pid] = $count;
    $captions[$picture->pid] = $picture->caption;
    $count++;
  }

  drupal_add_js(
    array(
      'gallerixManager' => array(
        'index' => $index,
        'pictures' => $pictures,
        'captions' => $captions,
        'count' => $count,
        'settings' => array(
          'deleteURL' => url('node/' . $node->nid . '/pictures/manage/delete/'),
          'submitURL' => url('node/' . $node->nid . '/pictures/manage/submit'),
          'reloadURL' => url('node/' . $node->nid . '/pictures', array('absolute' => TRUE)),
          'allowSorting' => $sort == 'custom',
        ),
      ),
    ),
    'setting'
  );

  // Let the submit function know what to process.
  $form['actions'] = array(
    '#type' => 'value',
    '#value' => $actions
  );

  $form['save'] = array(
    '#type' => 'button',
    '#value' => t('Save Changes'),
    '#attributes' => array('class' => 'gallerix-management-save'),
  );

  return $form;
}



/**
 * Themes the management form into a table and adds the action buttons.
 *
 */
function theme_gallerix_manage_pictures_form($form) {
  $node = node_load($form['node_id']['#value']);

  $upload_files = variable_get('gallerix_upload_number', 1);
  $paths = _gallerix_album_paths($node, TRUE);
  $save = drupal_render($form['save']);

  $sort = $node->gallerix['sort'];
  $sort = ($sort == 'default') ? variable_get('gallerix_default_sort', 'ascending') : $sort;

  $header = array(t('Picture'), t('Caption'), t('Actions'));
  $rows = array();

  $count = 0;

  foreach (element_children($form['gallerix_album']['files']) as $key) {
    $pid = $form['gallerix_album']['files'][$key]['#value']['pid'];
    $basename = $form['gallerix_album']['files'][$key]['#value']['basename'];

    $picture = gallerix_load_picture($pid);

    $thumbnail_attributes = array(
      'class' => 'gallerix-thumbnail-field',
      'id' => 'gallerix-thumbnail-field-' . $pid,
      'height' => '40',
      'style' => 'margin: 2px; cursor: pointer;'
    );

    $thumbnail = theme('image', $picture->thumbnail, $alt = '', $basename, $thumbnail_attributes, FALSE);

    unset($form['gallerix_album']['checkboxes'][$key]['#title']);
    unset($form['gallerix_album']['captions'][$key]['#title']);
    unset($form['gallerix_album']['checkboxes'][$key]);

    $actions = '';

    // add sort buttons?
    if ($sort == 'custom') {
      $top = theme('image', theme('gallerix_icon', 'top.png'), t('Top'), t('Click to shift to the top.'));
      $actions .= l($top, '#', array('attributes' => array('id' => 'gallerix-top-field-' . $pid, 'class' => 'gallerix-top-field  gallerix-action'),    'html' => TRUE));

      $up = theme('image', theme('gallerix_icon', 'up.png'), t('Up'), t('Click to shift up.'));
      $actions .= l($up, '#', array('attributes' => array('id' => 'gallerix-up-field-' . $pid, 'class' => 'gallerix-up-field  gallerix-action'),     'html' => TRUE));

      $down = theme('image', theme('gallerix_icon', 'down.png'), t('Down'), t('Click to shift down.'));
      $actions .= l($down, '#', array('attributes' => array('id' => 'gallerix-down-field-' . $pid, 'class' => 'gallerix-down-field  gallerix-action'),   'html' => TRUE));

      $bottom = theme('image', theme('gallerix_icon', 'bottom.png'), t('Bottom'), t('Click to shift to the bottom.'));
      $actions .= l($bottom, '#', array('attributes' => array('id' => 'gallerix-bottom-field-' . $pid, 'class' => 'gallerix-bottom-field  gallerix-action'), 'html' => TRUE));
    }

    // delete/restore buttons
    $delete = theme('image', theme('gallerix_icon', 'delete.png'), t('Delete'), t('Click to mark for deletion.'));
    $actions .= l($delete, '#', array('attributes' => array('id' => 'gallerix-delete-field-' . $pid, 'class' => 'gallerix-delete-field  gallerix-action'), 'html' => TRUE));

    $restore = theme('image', theme('gallerix_icon', 'restore.png'), t('Restore'), t('Click to restore this picture.'));
    $actions .= l($restore, '#', array('attributes' => array('id' => 'gallerix-restore-field-' . $pid, 'class' => 'gallerix-restore-field  gallerix-action', 'style' => 'display: none;'), 'html' => TRUE));

    $row_data = array(
      $thumbnail,
      drupal_render($form['gallerix_album']['captions'][$key]),
      $actions,
    );

    $rows[] = array(
      'data' => $row_data,
      'class' => 'gallerix-management-row',
      'id' => 'gallerix-management-row-' . $pid,
    );

    $count++;
  }

  $output = theme('gallerix_message_box');

  $output .= theme('gallerix_manage_pictures_help');
  $output .= $save;
  $output .= theme('table', $header, $rows);
  $output .= $save;

  return drupal_render($form) . $output;
}


function theme_gallerix_manage_pictures_help() {
  return '<p>'
    . t('Customize your album here. You can use the tab key to shift quickly between pictures.')
    . '<br />'
    . t('If you chose custom ordering, hold down the CTRL key while pressing up or down to reorder your pictures quickly.')
    . '<br />'
    . t('Remember to save changes!')
    . '</p>';
}


/**
 * Uses AJAX to handle form submission. Note that this is NOT the same as
 * gallerix_manage_pictures_form_submit() which is unused in this implementation.
 *
 * This function does not return.
 */
function gallerix_manage_form_submit() {
  global $user;

  cache_clear_all(NULL, 'cache_gallerix');

  // work on all the images, some will be deleted and
  // others are sorted and re-captioned
  $nid = 0;
  foreach ($_POST as $pid => $content) {
    $picture = gallerix_load_picture($pid);
    if (!empty($picture)) {
      if ($nid == 0) {
        $nid = $picture->nid;
      }
      elseif ($nid != $picture->nid) {
        // Hmmm...
        watchdog('gallerix', 'Incompatible picture nid reference for picture @picture (@pic_nid, expected @nid)',
          array('@picture' => $picture->pid, '@pic_nid' => $picture->nid, '@nid' => $nid), WATCHDOG_ERROR);
        continue;
      }
      // Unauthorized request. (Admin always fine, or administrators)
      if ($user->uid != 1 && $user->uid != $picture->uid && !user_access('administer album')) {
        watchdog('gallerix', 'Unauthorized picture modification attempt for picture @picture by @user (@uid)',
          array('@picture' => $pid, '@user' => $user->name, '@uid' => $user->uid), WATCHDOG_ERROR);

        $result = array(
          'status' => 'error',
          'message' => t("Unauthorized picture modification attempt. Your IP has been logged."),
        );
        print gallerix_to_js($result);
        exit();
      }

      $content = explode("|#|", $content);
      switch ($content[0]) {
      case 'sort':
        // Note that in the database sorting starts at 1, not 0.
        db_query("UPDATE {gallerix_pictures} SET caption = '%s', sort = %d WHERE pid = %s",
          trim($content[2]), $content[1] + 1, $pid);
        break;

      case 'delete':
        gallerix_delete_picture($pid);
        break;

      default:
        // hacker at work?
        watchdog('gallerix', 'Unknown action @action in management.inc',
          array('@action' => $content[0]), WATCHDOG_ERROR);
        break;

      }
    }
  }

  if ($nid == 0) {
    // this should never happen unless the request was really bad
    $result = array(
      'status' => 'error',
      'message' => t('No pictures could be loaded.'),
    );
  }
  else {
    $count = db_result(db_query("SELECT COUNT(nid) FROM {gallerix_pictures} WHERE nid = %d", $nid));
    if ($count) {
      $result = array(
        'status' => 'success',
        'message' => t('Pictures updated successfully.'),
      );
    }
    else {
      $result = array(
        'status' => 'reload',
        'message' => t('All pictures were deleted.'),
      );
    }
  }

  print gallerix_to_js($result);
  exit();
}




/**
 * Deletes a picture from an album through the management form.
 *
 */
function gallerix_manage_form_delete($pid) {
  global $user;

  $picture = gallerix_load_picture($pid);
  if (!empty($picture)) {
    // Unauthorized request. (Admin always fine, or album administrators)
    if ($user->uid != 1 && $user->uid != $picture->uid && !user_access('administer album')) {
      watchdog('gallerix', 'Unauthorized picture modification attempt for picture @picture by @user (@uid)',
        array('@picture' => $pid, '@user' => $user->name, '@uid' => $user->uid), WATCHDOG_ERROR);

      $result = array(
        'status' => 'error',
        'message' => t("Unauthorized picture modification attempt. Your IP has been logged."),
      );
      print gallerix_to_js($result);
      exit();
    }
    else {
      gallerix_delete_picture($pid);
    }
  }
}




// vim: ts=2 sw=2 et syntax=php
